package 
{
	import actors.Player;
	import actors.Thug;
	import actors.ThugB;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.net.SharedObject;
	import flash.net.URLRequest;
	import loot.Dollar;
	import loot.GunLarger;
	import loot.Heart;
	import utils.Assets;
	import utils.XMLLoader;

	/**
	 * [ ] Score (Coins?)
	 * [x] Restart game on death.
	 * [/] Thug aim and looks
	 * [x] Shoot a Thug
	 * [x] Thug shoot you
	 * 
	 * @author christoffer@enedahl.com
	 */
	[Frame(factoryClass="Preloader")]
	public class Main extends Sprite
	{
		private const FOLDER:String = "data/";
		
		public static var assets:Assets;
		public static var gameTime:int = 0;;
		public static var map:TileMap;
		public static var world:World;
		private static var menu:Menu;
		private static var _highScore:int;
		
		//Reference the classes we want in the game
		private function doNotRunJustImportForTheCompiler():void 
		{
			var a:Array = [Player, Thug, ThugB, Heart, Dollar, GunLarger];
		}
		
		public function Main():void 
		{
			if (stage) init();
			else addEventListener(Event.ADDED_TO_STAGE, init);
			
			loadScore();
		}
		
		private function loadScore():void 
		{
			var so:SharedObject = SharedObject.getLocal("highscore");
			
			try{
				_highScore = int( so.data.highscore );
			}catch (err:Error)
			{
				_highScore = 0;
			}
			
		}
		
		private function saveScore():void 
		{
			var so:SharedObject = SharedObject.getLocal("highscore");
			so.data.highscore = _highScore;
		}
		
		public static function get player():Player {
			return world.player;
		}

		private function init(e:Event = null):void 
		{
			removeEventListener(Event.ADDED_TO_STAGE, init);
			// entry point
			
			assets = new Assets();
			assets.complete.addOnce( onAssetsLoaded );
			assets.load( new URLRequest( FOLDER + "gfx/assets.swf" ));
			
		}
		
		private function onAssetsLoaded( pSuccess:Boolean ):void 
		{
			if ( pSuccess) {
				initMainMenu();
			}else {
				trace("Failed to load assets");
			}
		}
		
		private function initMainMenu():void 
		{
			menu = new Menu();
			menu.gfx.scaleX = menu.gfx.scaleY = 2;
			menu.newGame.add( onNewGame );
			
			showMainMenu( false );
		}
		
		private function showMainMenu( pNewScore:Boolean ):void 
		{
			addChild( menu.gfx );
			
			menu.setScore( _highScore, pNewScore);
		}
		
		private function onNewGame():void 
		{
			removeChild( menu.gfx );
			
			loadMaps();
		}
		
		private function loadMaps():void 
		{
			var loader:XMLLoader = new XMLLoader();
			loader.complete.addOnce( onMapsLoaded );
			loader.load( new URLRequest( FOLDER +"maps.xml") );
			
		}
		
		private function onMapsLoaded(pSuccess:Boolean, pMaps:XML ):void 
		{
			if ( pSuccess) {
				trace("loaded " + pMaps);
				
				for each (var map:XML in pMaps.map) 
				{
					var src:String = map.@src;
					var id:int = parseInt(map.@id);
					
					var loader:XMLLoader = new XMLLoader();
					loader.complete.addOnce( onMapLoaded );
					loader.load( new URLRequest( FOLDER + src ) );
				}
				
			}else {
				trace("Failed to load Maps");
			}
		}
		
		private function onMapLoaded( pSuccess:Boolean, pMap:XML ):void {
			
			if ( pSuccess) {
				map = new TileMap( FOLDER );
				map.complete.addOnce( onMapContentLoaded );
				map.loadXML( pMap );
				
			}else {
				trace("Failed to load Map");
			}
		}
		
		private function onMapContentLoaded( pMap: TileMap ):void 
		{
			if( world == null){
				createWorld( pMap );
			}
		}
		
		private function createWorld( pMap: TileMap ):void 
		{
			world = new World();
			addChild( world.gfx );
			world.setMap( pMap );
			world.gameOver.add( onGameOver );
		}
		
		private function onGameOver( pScore:int ):void 
		{
			var newRecord:Boolean = false;
			if ( _highScore <= pScore) {
				_highScore = pScore;
				saveScore();
				newRecord = true;
			}
			
			world.dispose();
			world = null;
			
			showMainMenu( newRecord );
		}
		
		

	}

}