﻿package  
{
	import com.nodename.utils.Enumeration;
	import adobe.utils.CustomActions;
	import flash.display.BitmapData;
	import flash.events.DataEvent;
	import flash.events.Event;
	import flash.geom.Point;
	import flash.geom.Rectangle;
	import flash.net.URLLoader;
	import flash.net.URLRequest;
	import flash.utils.ByteArray;
	import flash.utils.Dictionary;
	import org.osflash.signals.Signal;
	/**
	 * ...
	 * @author christoffer@enedahl.com
	 */
	public class TileMap
	{
		public var map:Array;
		public var mapBG:Array;
		public var textures:Vector.<BitmapData>;
		public var width:int;
		public var height:int;
		private var _layers:Vector.<Layer>;
		private var _tilesets:Vector.<TileSet>;
		private var _tileproperty:Dictionary;
		private var tilesWide:int;
		private var _isLoaded:Boolean = false;
		private var _connectionsByCoordinate:Dictionary;
		private var _layerTiles:Layer;
		private var _layerBackground:Layer;
		private var _folder:String;
		private var _complete:Signal = new Signal(TileMap);
		private var _spawn:Vector.<ObjectDefinition>;
		
		public function TileMap( pFolder:String ) 
		{
			_folder = pFolder;
			textures = new Vector.<BitmapData>();
			textures.push( null );
			
		}
		
		public function getTile(x:int, y:int ):Tile 
		{
			if ( x < 0) return null;
			if ( y < 0) return null;
			if ( x >= width) return null;
			if ( y >= height) return null;
			
			return map[y][x] as Tile;
		}
		
		public function getTileBG(x:int, y:int ):Tile 
		{
			if ( x < 0) return null;
			if ( y < 0) return null;
			if ( x >= width) return null;
			if ( y >= height) return null;
			
			return mapBG[y][x] as Tile;
		}
		
		public function setTile( pTile:Tile, x:int, y:int ):void 
		{
			if ( x < 0) return;
			if ( y < 0) return;
			if ( x >= width) return;
			if ( y >= height) return;
			
			map[y][x] = pTile;
		}
		
		public function get tileWidth():int 
		{
			return _tilesets[0].tilewidth;
		}
		
		public function get tileHeight():int 
		{
			return _tilesets[0].tileheight;
		}
		
		public function get isLoaded():Boolean { return _isLoaded; }
		
		public function get complete():Signal 
		{
			return _complete;
		}
		
		public function get spawn():Vector.<ObjectDefinition>
		{
			return _spawn;
		}
		
		public static function getXMLFromClass( pClass:Class ):XML {
			
			var file:ByteArray = new pClass;
			var str:String = file.readUTFBytes( file.length );
			return new XML( str );
		}
		
		public function loadXMLFromClass( pXmlClass:Class ):void {
			loadXML( getXMLFromClass( pXmlClass ) );
		}
		
		public function loadXML( pXml:XML ):void 
		{	
			_tilesets = new Vector.<TileSet>();
			var i:int = 0;
			for each (var tilesetNode:XML in pXml.tileset) 
			{
				_tilesets.push( new TileSet( tilesetNode, _folder ) );
				_tilesets[i].loaded.addOnce( onTilesetLoaded );
				i++;
			}
			
			_layers = new Vector.<Layer>();
			for each( var layerNode:XML in pXml.layer ) {
				var len:uint = _layers.push( new Layer(layerNode));
				switch ( _layers[len - 1].name ) 
				{
					case "Tiles":
						_layerTiles = _layers[len - 1];
						break;
						
					case "Background":
						_layerBackground = _layers[len - 1];
						break;
				}
			}
			
			//Load tileproperties
			_tileproperty = new Dictionary();
			for each (var tile:XML in pXml.tileset.tile) 
			{
				var key:int = parseInt( tile.@id ) + _tilesets[0].firstGid;
				
				try{
					var type:String = tile.properties.property.(@name == "type")[0].@value;
				}catch (e:Error){}
				
				var enumType:TilePropertyType = TilePropertyType.fromString(type);
			
				if ( enumType == null) {
					
					trace("Unknown type " + type);
					
					enumType = TilePropertyType.defaultValue;
					key = 0;
				}
				
				
				//value
				try{
					var value:Number = parseFloat( tile.properties.property.(@name == "value")[0].@value );
				}catch (e:Error) {
					value = NaN;
				}
				
				try{
					var spawnString:String = tile.properties.property.(@name == "spawn")[0].@value;
				}catch (e:Error) {
					spawnString = null;
				}
				
				var tileP:TileProperty = new TileProperty( enumType, key, value, spawnString);
				
				
				//name
				try{
					var name:String = tile.properties.property.(@name == "name")[0].@value;
					_tileproperty[name] = tileP; //Save by keyname also
				}catch (e:Error){}
				
				_tileproperty[ key ] = tileP;
				//trace("_tileproperty " + key +" = " + tileP );
			
				try{
					var show_as:String = tile.properties.property.(@name == "show_as")[0].@value;
					if ( show_as != null) {
						tileP.show_as = show_as;
					}
				}catch (e:Error){}
			}
			
			//Solve showas
			for each (var tileProp:TileProperty in _tileproperty) {
			
				if ( tileProp.show_as != null) {
					tileProp.textureGid = getTilePropertyByName( tileProp.show_as ).textureGid;
				}
				
			}
			
			//Load the map
			loadLayer( _layerTiles, _layerBackground );
			
			var connections:Dictionary = new Dictionary();
			_connectionsByCoordinate = new Dictionary();
			
			_spawn = new Vector.<ObjectDefinition>;
			
			for each( var objectNode:XML in pXml.objectgroup.object ) {
				
				var object:ObjectDefinition = new ObjectDefinition(objectNode);
				
				switch (object.type) 
				{
					case "source":
					case "target":
						var conn:TileConnection = connections[ object.name ];
						if ( conn == null) {
							conn = new TileConnection( object.name );
							connections[ object.name ] = conn;
						}
						
						if( type == "source"){
							conn.addSource( object.position );
						}else{
							conn.addTarget( object.position );
						}
						
						var ix:int = object.x >> 5;
						var iy:int = object.y >> 5;
						
						_connectionsByCoordinate[ ix +"x" + iy ] = conn;
						break;
						
					case "spawn":
						trace("Spawn " + object.name +" at " + object.position );
						_spawn.push( object );
						break;
						
					default:
						trace ("Unknown connection type: " + type);
						break;
				}
				
				
			}
		}
		
		public function getConnectionAt(pX:int, pY:int):TileConnection {
			return _connectionsByCoordinate[ pX +"x" + pY ] as TileConnection;
		}
		
		public function getTileProperty( id:int ):TileProperty 
		{
			var tileProp:TileProperty = _tileproperty[ id ] as TileProperty;
			//trace("GetTileProperty " + id +" = " + tileProp);
			return tileProp;
		}
		
		public function getTilePropertyByName( pName:String ):TileProperty 
		{
			return _tileproperty[ pName ] as TileProperty;
		}
		
		private function loadLayer( pLayer:Layer, pBackgroundLayer:Layer ):void 
		{
			mapBG = new Array();
			if ( pBackgroundLayer != null) {
				fillMap(pBackgroundLayer, mapBG );	
			}
			
			map = new Array();
			if (pLayer != null) {
				fillMap(pLayer, map );
			}
			
		}
		
		private function fillMap(pLayer:Layer, pMap:Array):void 
		{
			width = pLayer.width;
			height = pLayer.height;
			for (var j:int = 0; j < pLayer.height; j++) {
				
				var row:Array = new Array();
				for (var i:int = 0; i < pLayer.width; i++) 
				{
					var tile:Tile = new Tile();
					tile.textureGid = pLayer.getTextureGid( i,j );
					tile.tileProperty = getTileProperty( tile.textureGid );
					
					row.push( tile );
				}
				pMap.push( row );
			}
		}
		
		
		private function onTilesetLoaded( pTileSet:TileSet ):void
		{
			var allTilesAreLoaded:Boolean = true;
			for (var i:int = 0; i < _tilesets.length; i++) 
			{
				if ( ! _tilesets[i].isLoaded ) {
					allTilesAreLoaded = false;
					break;
				}
			}
			
			//Fill textures
			tilesWide = int( pTileSet.bitmapData.width / pTileSet.tilewidth );
			var tilesHigh:int = int( pTileSet.bitmapData.height / pTileSet.tileheight );
			
			var gid:int = pTileSet.firstGid;
			
			var tileRect:Rectangle = new Rectangle(0, 0, pTileSet.tilewidth, pTileSet.tileheight);
			var dest:Point = new Point();
			
			for (var y:int = 0; y < tilesHigh; y++)
			{ 
				for (var x:int = 0; x < tilesWide; x++) 
				{
					var tile:BitmapData = new BitmapData(pTileSet.tilewidth, pTileSet.tileheight);
					
					var source:Rectangle = tileRect.clone();
					source.offset( x * pTileSet.tilewidth, y * pTileSet.tileheight);
					tile.copyPixels( pTileSet.bitmapData, source, dest);
					textures[gid] = tile;
					gid++;
					
				}
			}
			
			_isLoaded = true;
			
			if ( allTilesAreLoaded ) {
				trace("Start the game! all tiles are loaded");
				
				_complete.dispatch(this);
			}
			
		}
		
		public function getTexture( pX:int, pY:int):BitmapData
		{
			return textures[ pX + (pY * tilesWide)];
		}
		
	}

}