package  
{
	import actors.Actor;
	import actors.Player;
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.geom.Point;
	import flash.geom.Rectangle;
	import flash.utils.getDefinitionByName;
	import flash.utils.getTimer;
	import loot.Heart;
	import loot.ILoot;
	import org.osflash.signals.Signal;
	import utils.Factory;
	/**
	 * ...
	 * @author christoffer@enedahl.com
	 */
	public class World 
	{
		private var _gfx:Sprite;
		private var _camera:Camera;
		private var _input:Input;
		
		private var _currentLevel:TileMap;
		private var _textures:Vector.<BitmapData>;
		private var _screenTileWidth:int;
		private var _screenTileHeight:int;
		private var _gameCanvasData:BitmapData;
		private var _gameCanvas:Bitmap;
		private var _gameRect:Rectangle;
		private var _tile:Rectangle;
		private var _renderPoint:Point = new Point();
		
		private var _objects:Vector.<IObject>;
		private var _factory:Factory;
		private var _player:Player;
		private var _lastFrameTime:int;
		private var _gameTime:int;
		private var _viewWidth:Number;
		private var _viewHeight:Number;
		private var _loot:Vector.<IObject>;
		private var _hud:Hud;
		private var _pause:Boolean = false;
		private var _pauseMenu:PauseMenu;
		private var _gameOver:Signal = new Signal(int);
		
		public function World() 
		{
			_gfx = new Sprite();
			_gfx.addEventListener(Event.ADDED_TO_STAGE, onAddedToStage);
			
			_factory = new Factory();
		}
		
		private function onAddedToStage(e:Event):void 
		{
			_gfx.removeEventListener(Event.ADDED_TO_STAGE, onAddedToStage);
			
			//Double size
			_gfx.scaleX = _gfx.scaleY = 2;
			_viewWidth = _gfx.stage.stageWidth >> 1;
			_viewHeight = _gfx.stage.stageHeight >> 1;
			
			_camera = new Camera(_viewWidth, _viewHeight);
			_gameRect = new Rectangle(0, 0, _viewWidth, _viewHeight);
			_gameCanvasData = new BitmapData( _viewWidth, _viewHeight, false, 0x000000);
			_gameCanvas = new Bitmap(_gameCanvasData);
			
			_gfx.addChild( _gameCanvas );
			if (_currentLevel == null) return;
			
			init();
		}
		
		public function setMap(pMap:TileMap):void 
		{
			_currentLevel = pMap;
			_textures = _currentLevel.textures;
			_screenTileWidth = int(_gfx.stage.stageWidth / _currentLevel.tileWidth)+2;
			_screenTileHeight = int(_gfx.stage.stageHeight / _currentLevel.tileHeight)+2;
			_tile = new Rectangle(0,0, _currentLevel.tileWidth, _currentLevel.tileHeight);
			
			if ( _gfx.stage == null) return;
			
			init();
		}
		
		public function walkOnTile(tile:Tile, x:int, y:int):void 
		{
			if ( tile == null || tile.tileProperty == null ) return;
				
			if ( tile.tileProperty.type == TilePropertyType.CRATE ) {
				
				var spawnName:String = tile.tileProperty.spawn;
				var foundSomething:Boolean = false;
				
				var spawnedObject:IObject = _factory.spawnByName( spawnName, x, y - 32 ) as IObject;
				if( spawnedObject != null)
				{
					addObject( spawnedObject );
					Main.assets.newSound("Loot").play();
					
				}else {
					Main.assets.newSound("FoundNothing").play();
				}
				
				var openCrate:TileProperty = Main.map.getTilePropertyByName( TilePropertyType.CRATE_OPEN.toString() );
				
				tile.tileProperty = openCrate;
			}
		}
		
		private function init():void 
		{
			_gameTime = 0;
			_lastFrameTime = getTimer();
			_input = new Input(_gfx.stage);
			_gfx.stage.addEventListener(Event.ENTER_FRAME, onFrameLoop );
			
			_objects = new Vector.<IObject>(); //Spawn!
			_loot = new Vector.<IObject>();
			for each(var object:ObjectDefinition in _currentLevel.spawn) {
				addObject( _factory.spawn( object ) as IObject );
			}
			
			_hud = new Hud();
			
			_gfx.addChild( _hud.gfx );
		}
		
		private function addObject( spawnedObject:IObject ):void 
		{
			if ( spawnedObject == null ) return;
			if ( spawnedObject is Actor) {
				Actor(spawnedObject).spawn.add( onSpawn );
				
				if ( spawnedObject is Player) {
					_player = spawnedObject as Player;
					_camera.setTarget( _player as IObject, true );
				}
			}
			
			if ( spawnedObject is ILoot ) {
				_loot.push( spawnedObject );
			}
			
			_objects.push( spawnedObject );
			_gfx.addChild( spawnedObject.gfx );
		}
		
		private function onSpawn( pSpawnee:IObject, pSpawnObject:IObject ):void 
		{
			pSpawnObject.pos.x = pSpawnee.pos.x;
			pSpawnObject.pos.y = pSpawnee.pos.y - 16;
			
			addObject( pSpawnObject );
		}
		
		private function onFrameLoop(e:Event):void 
		{
			//Time
			var now:int = getTimer();
			var frameTime:int = now - _lastFrameTime;
			_lastFrameTime = now;
			
			if ( _input.getKeyWasReleased( Input.ESC ) || _input.getKeyWasReleased( Input.P ) ) {
				pause = !pause;
			}
			
			if ( !pause ){
				//pause? do not add to _gameTime
				Main.gameTime += frameTime;
				
				//Input update
				if ( _player != null ) {
					if (_player.isDead ) {
						if ( _player.removeNextUpdate ) {
							
							_gameOver.dispatch( _player.dollar );
							
							return;
						}
					}else{
						_player.updateInput( _input );
					}
				}
				
				//Object update
				for (var i:int = _objects.length - 1; i >= 0; i--) 
				{
					var object:IObject = _objects[i];
					
					if ( object.removeNextUpdate ) {
						_objects.splice(i, 1);
						
						if ( object is ILoot) {
							removeLoot( object );
						}
						
						object.dispose();
					}else{
						object.update( frameTime );
					}
				}
				
				render();
			}
			
			_input.nextFrame();
		}
		
		private function removeLoot(object:IObject):void 
		{
			for (var i:int = 0; i < _loot.length; i++) 
			{
				if ( _loot[i] == object) {
					_loot.splice(i, 1);
					return;
				}
			}
		}
		
		private function render():void
		{
			_camera.updateSpring();
			_gameCanvasData.fillRect( _gameRect, 0x000000);
		
			var ix:int;
			var iy:int;
			var cx:int = int( _camera.x / _tile.width );
			var cy:int = int( _camera.y / _tile.height );
			
			var playerPos:Point = new Point( (_player.pos.x >> 5) << 5 , (_player.pos.y + _player.collisionRect.top >> 5) << 5 );
			var visiblePart:Rectangle = new Rectangle(cx, cy, _screenTileWidth, _screenTileHeight);
			
			var camPoint:Point = _camera.point;
			
			for ( iy = cy; iy < cy+_screenTileHeight; iy++){				
				var yOffset:int = iy * _tile.height;
				for ( ix = cx-1; ix < cx + _screenTileWidth; ix++) {
			
					var tile:Tile = _currentLevel.getTile(ix, iy);
					
					if ( tile != null) {
					
						//if ( _freeMode || tile.beenSeen ) 
						{
							_renderPoint.x = (ix * _tile.width) - int(camPoint.x);
							_renderPoint.y = yOffset - int(camPoint.y);
							
							var tileBg:Tile = _currentLevel.getTileBG(ix, iy);
							
							if( tileBg.textureGid > 0 ){
								_gameCanvasData.copyPixels( _textures[ tileBg.textureGid ], _tile, _renderPoint);
							}
							
							if( tile.textureGid > 0 ){
								_gameCanvasData.copyPixels( _textures[ tile.textureGid ], _tile, _renderPoint);
							}
							
						}
					}
					
				}
			}
			
			for each(var object:IObject in _objects) {
				object.render( _gameCanvasData, camPoint );
			}
			
			//_gameCanvasData.copyPixels( _textures[ 10 ], _tile, new Point(playerPos.x -camPoint.x, playerPos.y -camPoint.y) );
			
		
		}
		
		public function get gfx():Sprite 
		{
			return _gfx;
		}
		
		public function get objects():Vector.<IObject> 
		{
			return _objects;
		}
		
		public function get viewWidth():Number 
		{
			return _viewWidth;
		}
		
		public function get viewHeight():Number 
		{
			return _viewHeight;
		}
		
		public function get player():Player 
		{
			return _player;
		}
		
		public function get loot():Vector.<IObject> 
		{
			return _loot;
		}
		
		public function get pause():Boolean 
		{
			return _pause;
		}
		
		public function set pause(value:Boolean):void 
		{
			if ( _pause == value ) return;
			_pause = value;
			
			if ( _pause ) {
				if ( _pauseMenu == null) {
					_pauseMenu = new PauseMenu();
					_pauseMenu.gfx.alpha = 0.8;
					_pauseMenu.play.addOnce( onPauseMenuPlay );
					_gfx.addChild( _pauseMenu.gfx );
				}
			}else {
				if ( _pauseMenu != null) {
					_pauseMenu.dispose();
					_pauseMenu = null;
				}
			}
		}
		
		public function get gameOver():Signal 
		{
			return _gameOver;
		}
		
		private function onPauseMenuPlay():void 
		{
			pause = false;
		}
		
		public function dispose():void 
		{
			if ( _pauseMenu != null) {
				_pauseMenu.dispose();
				_pauseMenu = null;
			}
			
			try{
				_gfx.stage.removeEventListener(Event.ENTER_FRAME, onFrameLoop );
			}catch (err:Error){}
			
			if ( _gfx != null) {
				if ( _gfx.parent != null) {
					_gfx.parent.removeChild( _gfx);
				}
				_gfx = null;
			}
			
			if ( _camera!= null) {
				_camera.dispose();
				_camera = null
			}
			
			_input = null;
			
			_currentLevel = null;
			_textures = null;
			_gameCanvasData = null;
			_gameCanvas= null;
			_gameRect= null;
			_tile= null;
			_renderPoint= null;
			
			if( _objects != null){
				for (var i:int = _objects.length - 1; i >= 0; i--) 
				{
					_objects[i].dispose()
				}
				_objects = null;
			}
			
			if( _loot != null){
				for ( i = _loot.length - 1; i >= 0; i--) 
				{
					_loot[i].dispose();
				}
				_loot = null;
			}
			
			_factory = null;
			_player = null;
			
			if( _hud != null){
				_hud.dispose();
				_hud = null;
			}
			
			if ( _gameOver!= null) {
				_gameOver.removeAll();
				_gameOver = null
			}
		}
	}

}