package actors 
{
	import behaviours.BehaviourCollision;
	import behaviours.IBehaviour;
	import flash.display.BitmapData;
	import flash.display.DisplayObject;
	import flash.display.MovieClip;
	import flash.geom.Point;
	import flash.geom.Rectangle;
	import flash.media.Sound;
	import org.osflash.signals.Signal;
	import weapons.Bullet;
	/**
	 * ...
	 * @author christoffer@enedahl.com
	 */
	public class Actor implements IObject
	{
		private var _currentFrameObject:Object;
		protected var _collisionRect:Rectangle;
		protected var _gfx:MovieClip;
		protected var _pos:Point;
		protected var _spawn:Signal = new Signal(IObject , IObject);
		protected var _removeNextUpdate:Boolean = false;
		protected var _vel:Point;
		protected var _collision:IBehaviour;
		
		protected var _lastFireTime:int = 0;
		protected var _fireRate:int = 400;
		
		protected var _gunSize:int = 1;
		
		
		public function Actor( pGfx:MovieClip ) 
		{
			_gfx = pGfx;
			_pos = new Point();
			_vel = new Point();
			
			_collisionRect = new Rectangle(-8, -16, 16, 16);
			
			_collision = new BehaviourCollision( Main.map );
		}
		
		protected function shoot( ):Boolean 
		{
			if ( _lastFireTime + _fireRate > Main.gameTime) return false; 
			
			_lastFireTime = Main.gameTime;
			
			var range:int = 300 + ( _gunSize * 100);
			
			var bullet:Bullet = new Bullet( Main.assets.newMovieClip("MCBullet"), this, range );
			bullet.vel.x = (10*_gfx.scaleX);
			_spawn.dispatch( this, bullet);
		
			return true;
		}
		
		public function dispose():void {
		
			if ( _gfx != null) {
				if ( _gfx.parent != null) {
					_gfx.parent.removeChild( _gfx);
				}
				_gfx = null;
			}
			
			if ( _spawn!= null) {
				_spawn.removeAll();
				_spawn = null;
			}
			
			if ( _collision!= null) {
				_collision.dispose();
				_collision = null;
			}
			
		}
		
		public function gotoAndPlay( frame:Object, forceRestart:Boolean = false):void
		{
			if ( _currentFrameObject == frame && !forceRestart) return;
			
			_currentFrameObject = frame;
			_gfx.gotoAndPlay(frame);
			
		}
		
		public function setDefinition(object:ObjectDefinition):void 
		{
			pos.x = object.x;
			pos.y = object.y;
		}
		
		public function get pos():Point 
		{
			return _pos;
		}
		
		public function update( pLastFrameTime:int ):void 
		{
			if ( _collision != null) {
				_collision.update(this);
			}
		}
		
		public function render(dest:BitmapData, cameraPoint:Point):void 
		{
			_gfx.x = int(_pos.x - cameraPoint.x);
			_gfx.y = int(_pos.y - cameraPoint.y);
			
			//dest.setPixel32(_pos.x + _collisionRect.left  - cameraPoint.x, _pos.y + _collisionRect.top    - cameraPoint.y, 0xFF0000);
			//dest.setPixel32(_pos.x + _collisionRect.right - cameraPoint.x, _pos.y + _collisionRect.bottom - cameraPoint.y, 0xFF0000);
		}
		
		public function get gfx():DisplayObject 
		{
			return _gfx;
		}
		
		public function get spawn():Signal 
		{
			return _spawn;
		}
		
		public function get removeNextUpdate():Boolean 
		{
			return _removeNextUpdate;
		}
		
		public function set removeNextUpdate(value:Boolean):void 
		{
			_removeNextUpdate = value;
		}
		
		public function get collisionRect():Rectangle { return _collisionRect; }
		
		public function get vel():Point 
		{
			return _vel;
		}
		
		public function get collision():IBehaviour 
		{
			return _collision;
		}
		
		public function set collision(value:IBehaviour):void 
		{
			_collision = value;
		}
		
		public function get currentFrameObject():Object 
		{
			return _currentFrameObject;
		}
	}

}