package actors 
{
	import flash.events.Event;
	import flash.geom.Rectangle;
	import flash.media.Sound;
	import flash.media.SoundChannel;
	import loot.Dollar;
	import loot.ILoot;
	import org.osflash.signals.Signal;
	import weapons.Bullet;
	/**
	 * ...
	 * @author christoffer@enedahl.com
	 */
	public class Player extends Actor implements ISpawnable, IShootable
	{
		public var maxHP:int = 4;
		private var _speed:Number = 5.0;
		private var _hp:int = 3;
		private var _dollar:int = 0;
		private var _hitRect:Rectangle;
		private var _changedHP:Signal = new Signal(Player);
		private var _lowHPSound:SoundChannel;
		private var _changedDollar:Signal = new Signal(Player);
		
		protected var _shoot:Sound;
		public function Player()
		{
			super( Main.assets.newMovieClip("MCChar") );
			
			_hitRect = new Rectangle( -8, -32, 16, 32);
			
			
			_shoot = Main.assets.newSound("Shoot2");
			
			
			gotoAndPlay("idle");
		}
		
		
		override protected function shoot():Boolean 
		{
			if( super.shoot() ){
				_shoot.play();
				return true;
			}
			return false;
		}
		
		public function updateInput(input:Input):void 
		{
			var inputX:Number = input.getX() * _speed;
			var inputY:Number = input.getY() * _speed;
			
			_vel.x += inputX;
			_vel.y += inputY;
			
			if ( inputX < 0) {
				_gfx.scaleX = -1;
			}else if ( inputX > 0) {
				_gfx.scaleX = 1;
			}
			
			if ( currentFrameObject == "walk" || currentFrameObject == "idle") {
				if ( inputX == 0 && inputY == 0 ) {
					gotoAndPlay("idle");
				}else {
					gotoAndPlay("walk");
				}
			}
			
			if ( input.getKeyWasPressed( Input.Z )) {
				shoot();
			}
			
			
			if ( input.getKeyWasPressed( Input.DELETE )) {
				takeDamage(1);
			}
		}
		
		
		
		override public function update(pLastFrameTime:int):void 
		{
			var tx:int = pos.x >> 5;
			var ty:int = pos.y >> 5;
			var tile:Tile = Main.map.getTile( tx, ty );
			
			Main.world.walkOnTile( tile , pos.x , pos.y );
			
			var lootObjects:Vector.<IObject> = Main.world.loot;
			for each (var object:IObject in lootObjects) 
			{
				if ( object.collisionRect.containsPoint( pos.subtract( object.pos ) ) ) {
					var lootObject:ILoot = object as ILoot;
					lootObject.pickup( this );
				}
			}
			
			super.update(pLastFrameTime);
		}
		
		public function hit(bullet:Bullet):void 
		{
			takeDamage(bullet.damage);
		}
		
		private function takeDamage(pDamage:int):void 
		{
			hp -= pDamage;
			
			if ( hp <= 0) {
				die();
			}else {
				
				var dropDollars:int = int(dollar * .1); //Loose 10 %, max 10 dollars
				if (dropDollars < 0) dropDollars = 1;
				if (dropDollars > 10) dropDollars = 10;
				
				while ( dollar > 0 && dropDollars > 0) {
					dollar--;
					dropDollars--;
					_spawn.dispatch( this, new Dollar());
				}
				
				
				Main.assets.newSound("PlayerHit").play();
				gotoAndPlay("hit", true);
				_gfx.addEventListener("animation_complete_hit", onAnimationCompleteHit);
			}
		}
		
		private function die():void 
		{
			
			//TODO Play die animation!
			Main.assets.newSound("PlayerDie").play();
			
			gotoAndPlay("die");
			_gfx.addEventListener("animation_complete_die", onAnimationCompleteDie );
			
		}
		
		private function onAnimationCompleteHit(e:Event):void 
		{
			_gfx.removeEventListener("animation_complete_hit", onAnimationCompleteHit);
			gotoAndPlay("idle");
		}
		
		
		private function onAnimationCompleteDie(e:Event):void 
		{
			_gfx.removeEventListener("animation_complete_die", onAnimationCompleteDie );
			
			_removeNextUpdate = true;
			
		}
		
		public function get hitRect():Rectangle 
		{
			return _hitRect;
		}
		
		public function get isDead():Boolean 
		{
			return (_hp <= 0);
		}
		
		public function get hp():int 
		{
			return _hp;
		}
		
		public function set hp(value:int):void 
		{
			if ( isDead ) return;
			
			if ( value > maxHP) {
				value = maxHP;
			}
			_hp = value;
			
			if ( _hp == 1) {
				if ( _lowHPSound == null) {
					_lowHPSound = Main.assets.newSound("LowHP").play(0, int.MAX_VALUE);
				}
			}else {
				if ( _lowHPSound != null) {
					_lowHPSound.stop();
					_lowHPSound = null;
				}
			}
			
			_changedHP.dispatch(this);
		}
		
		public function get changedHP():Signal 
		{
			return _changedHP;
		}
		
		public function get dollar():int 
		{
			return _dollar;
		}
		
		public function set dollar(value:int):void 
		{
			_dollar = value;
			
			_changedDollar.dispatch(this);
		}
		
		public function get changedDollar():Signal 
		{
			return _changedDollar;
		}
		
		override public function dispose():void 
		{
			if ( _gfx != null ) {
				if( _gfx.hasEventListener("animation_complete_die") ){
					_gfx.removeEventListener("animation_complete_die", onAnimationCompleteDie );
				}
				
				if( _gfx.hasEventListener("animation_complete_hit") ){
					_gfx.removeEventListener("animation_complete_hit", onAnimationCompleteHit );
				}
			}
			
			if ( _changedHP!= null) {
				_changedHP.removeAll();
				_changedHP = null
			}
			
			if ( _lowHPSound != null) {
				_lowHPSound.stop();
				_lowHPSound = null;
			}
			
			_shoot = null;
			
			super.dispose();
		}
		
		public function upgradeGun(size:int):void 
		{
			_gunSize = size;
			_gfx.gun.gotoAndStop(2);
			
		}
		
		
	}

}