package actors 
{
	import flash.display.MovieClip;
	import flash.events.Event;
	import flash.geom.Rectangle;
	import loot.Dollar;
	import weapons.Bullet;
	/**
	 * ...
	 * @author christoffer@enedahl.com
	 */
	public class Thug extends Actor implements ISpawnable, IShootable
	{
		protected var _bulletInterval:int = 1000;
		
		private var _isDead:Boolean = false;
		protected var _hp:int = 3;
		private var _lastBulletTime:int;
		protected var _threatArea:int = 32*6;
		
		public function Thug( pGfx:MovieClip = null )
		{
			if ( pGfx == null) {
				pGfx = Main.assets.newMovieClip("MCThug") ;
			}
			
			super( pGfx );
			
			_fireRate = 1000;
			_collisionRect = new Rectangle( -8, -32, 16, 32);
			
			gotoAndPlay("idle");
		}
		
		override public function update(pLastFrameTime:int):void 
		{
			//Actions
			if ( !_isDead) {
				if ( currentFrameObject != "hit") { //If not in hit animation.
					
					aggressive();
				}else {
					_lastBulletTime = Main.gameTime + _bulletInterval;
				}
			}
			
			super.update(pLastFrameTime);
		}
		
		private function aggressive():void 
		{
			var player:Player = Main.player;
			var dX:int = player.pos.x - pos.x;
			if ( dX < -_threatArea ) return;
			if ( dX > _threatArea ) return;
			
			var dY:int = player.pos.y - pos.y;
			
			if ( dY < -_threatArea ) return;
			if ( dY > _threatArea ) return;
			
			if( dX < 0){
				_gfx.scaleX = -1;
			}else if (dX > 0) {
				_gfx.scaleX = 1;
			}
			
			shoot();
		}
		
		/*private function shoot():void 
		{
			var now:int = Main.gameTime;
			
			if ( _lastBulletTime < now) {
				//Shoot
				_lastBulletTime = now + _bulletInterval;
				var bullet:Bullet = new Bullet( Main.assets.newMovieClip("MCBullet"), this);
				bullet.vel.x = (10 * _gfx.scaleX);
				_spawn.dispatch( this, bullet);
			}
		}*/
		
		public function die():void 
		{
			if ( _isDead ) return;
			
			_isDead = true;
			
			for (var i:int = 0; i < 2; i++)
			{
				var dollar:Dollar = new Dollar();
				_spawn.dispatch(this, dollar);
			}
			
			gotoAndPlay("die");
			_gfx.addEventListener("animation_complete_die", onAnimationCompleteDie);
			
			//Main.assets.newSound("ThugDie").play();
			Main.assets.newSound("Hit2").play();
		}
		
		public function hit(bullet:Bullet):void 
		{
			_hp -= bullet.damage;
			if ( _hp <= 0) {
				die();
			}else {
				
				var dollar:Dollar = new Dollar();
				_spawn.dispatch(this, dollar);
				
				gotoAndPlay("hit", true);
				Main.assets.newSound("Hit").play();
				
				_gfx.addEventListener("animation_complete_hit", onAnimationCompleteHit);
			}
		}
		
		private function onAnimationCompleteHit(e:Event):void 
		{
			_gfx.removeEventListener("animation_complete_hit", onAnimationCompleteHit);
			gotoAndPlay("idle");
		}
		
		private function onAnimationCompleteDie(e:Event):void 
		{
			_gfx.removeEventListener("animation_complete_die", onAnimationCompleteDie);
			removeNextUpdate = true;
		}
		
		public function get isDead():Boolean 
		{
			return _isDead;
		}
		
		override public function dispose():void 
		{
			
			if( _gfx.hasEventListener("animation_complete_die" )){
				_gfx.removeEventListener("animation_complete_die", onAnimationCompleteDie);
			}
			
			if( _gfx.hasEventListener("animation_complete_hit" )){
				_gfx.removeEventListener("animation_complete_hit", onAnimationCompleteHit);
			}
			
			super.dispose();
		}
		
		public function get hitRect():Rectangle 
		{
			return _collisionRect;
		}
		
	}

}