package behaviours 
{
	import flash.geom.Point;
	import flash.geom.Rectangle;
	/**
	 * ...
	 * @author christoffer@enedahl.com
	 */
	public class BehaviourCollision implements IBehaviour
	{
		private var _map:TileMap;
		public var drag:Number = 0.5;
		
		public function BehaviourCollision( pMap:TileMap ) {
			_map = pMap;
		}
		
		public function update(pObject:IObject):void
		{
			var pos:Point = pObject.pos;
			var vel:Point = pObject.vel;
			var collisionRect:Rectangle = pObject.collisionRect;
			
			vel.x *= drag;
			vel.y *= drag;
			
			var dir:Point = new Point();
			
			if ( vel.x < 0) {
				//Going left
				var xStart:int = pos.x + collisionRect.left;
				var xEnd:int = pos.x + vel.x + collisionRect.left;
				var x1:int = xStart >> 5;
				var x2:int = xEnd >> 5;
				dir.x = -1;
				
			}else {
				//Going right
				xStart = pos.x + collisionRect.right;
				xEnd = pos.x + vel.x + collisionRect.right;
				x1 = xStart >> 5;
				x2 = xEnd >> 5;
				dir.x = 1;
			}
			
			if( x1 != x2) //X movement
			{
				for (var iy:int = (pos.y + collisionRect.top + 1 ) >> 5; iy <= (pos.y + collisionRect.bottom-1 ) >> 5; iy++) 
				{
					var tile:Tile = _map.getTile( x2, iy);
					if ( tile == null || tile.isWall ) {
						
						if ( dir.x == 1) {
							var nonCollisionPosX:int = (x2 << 5) - dir.x; //First pixel not in wall
							pos.x = nonCollisionPosX - collisionRect.right;
						}else {
							nonCollisionPosX = ((x2- dir.x) << 5) ; //First pixel not in wall
							pos.x = nonCollisionPosX - collisionRect.left;
						}
						
						vel.x = 0;
						break;
					}
				}
			}
			
			if ( vel.y < 0) {
				//Going up
				var yStart:int = pos.y + collisionRect.top;
				var yEnd:int = pos.y + vel.y + collisionRect.top;
				var y1:int = yStart >> 5;
				var y2:int = yEnd >> 5;
				dir.y = -1;
				
			}else {
				//Going down
				yStart = pos.y + collisionRect.bottom;
				yEnd = pos.y + vel.y + collisionRect.bottom;
				y1 = yStart >> 5;
				y2 = yEnd >> 5;
				dir.y = 1;
			}
			
			if( y1 != y2) //Y movement
			{
				for (var ix:int = (pos.x+ collisionRect.left + vel.x) >> 5; ix <= (pos.x+ collisionRect.right + vel.x) >> 5; ix++) 
				{
					tile = _map.getTile( ix, y2);
					if ( tile == null || tile.isWall ) {
						
						if ( dir.y == 1) {
							var nonCollisionPosY:int = (y2 << 5) - dir.y; //First pixel not in ground
							pos.y = nonCollisionPosY - collisionRect.bottom;
							vel.y = 0;
						}else {
							nonCollisionPosY = ((y2 - dir.y) << 5) ; //First pixel not in ceiling
							pos.y = nonCollisionPosY - collisionRect.top;
							vel.y = 0;
						}
						
						break;
					}
				}
			}
			
			pos.x += vel.x;
			pos.y += vel.y;
			
		}
		
		public function dispose():void
		{
			_map = null;
		}
		
	}

}