package loot 
{
	import actors.Player;
	import behaviours.BehaviourCollision;
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.MovieClip;
	import flash.geom.Point;
	import flash.geom.Rectangle;
	import loot.Heart;
	
	/**
	 * ...
	 * @author christoffer@enedahl.com
	 */
	public class BaseLoot extends BaseObject implements ILoot, ISpawnable
	{
		protected var _collision:BehaviourCollision;
		protected var _timeOfRemoval:int;
		protected var _timeOfSpawn:int;
		protected var _velZ:Number = 0;
		protected var _z:Number;
		
		public function BaseLoot( lootName:String, timeToLive:int = 7000 )
		{
			var gid:int = Main.map.getTilePropertyByName( lootName ).textureGidOriginal;
			var bitmapGfx:MovieClip = new MovieClip();
			bitmapGfx.addChild( new Bitmap( Main.map.textures[gid].clone() ) );
			
			super( bitmapGfx );
			
			_collisionRect = new Rectangle(0, 0, 32, 32);
			
			_collision = new BehaviourCollision( Main.map );
			_collision.drag = 0.95;
			
			_timeOfSpawn = Main.gameTime;
			_timeOfRemoval = _timeOfSpawn + timeToLive;
			
			var angle:Number = Math.random() * Math.PI * 2;
			var length:Number = Math.random() * 3;
			
			_vel.x = Math.cos( angle ) * length;
			_vel.y = Math.sin( angle ) * length;
			
			_velZ = -13 + (Math.random()*6);
			_z = 0;
			
		}
		
		override public function update(pLastFrameTime:int):void 
		{
			if( _collision != null){
				_collision.update( this );
			}
			
			updateZ(this);
			
			super.update(pLastFrameTime);
			
			if ( _timeOfRemoval < Main.gameTime ) {
				removeNextUpdate = true;
			}
		}
		
		override public function render(dest:BitmapData, cameraPoint:Point):void 
		{
			_gfx.x = _pos.x - cameraPoint.x;
			_gfx.y = _pos.y + _z - cameraPoint.y;
			
			var blinksLeft:int = int( (_timeOfRemoval - Main.gameTime) / 100 );
			
			if ( blinksLeft < 30 ) {
				_gfx.visible = (blinksLeft % 2 == 0);
			}
		}
		
		private function updateZ( object:ILoot ):void 
		{
			var nextZ:Number = object.Z + object.velZ;
			if ( nextZ > 0 ) {
				object.velZ *= -0.6;
				object.Z = 0;
			}else {
				object.Z = nextZ;
			}
			object.velZ += 0.98;
		}
		
		override public function dispose():void 
		{
			if ( _collision!= null) {
				_collision.dispose();
				_collision = null
			}
			
			super.dispose();
		}
		
		/*public function pickup(player:Player):void 
		{
			if ( _timeOfSpawn + 1000 > Main.gameTime) return; //Cannot pick this up the first second
			
			if ( _z > -32) {
				player.hp ++;
				Main.assets.newSound("FoundHeart").play();
				removeNextUpdate = true;
			}
		}*/
		
		public function setDefinition(object:ObjectDefinition):void 
		{
			pos.x = object.x;
			pos.y = object.y;
		}
		
		public function pickup(player:Player):void 
		{
			trace("TODO to override pickup");
		}
		
		public function get Z():Number 
		{
			return _z;
		}
		
		public function set Z(value:Number):void 
		{
			_z = value;
		}
		
		public function get velZ():Number 
		{
			return _velZ;
		}
		
		public function set velZ(value:Number):void 
		{
			_velZ = value;
		}
		
	}

}